@@ -10,20 +10,19 @@
 import builtins
 import datetime
 from enum import Enum
-import glob
 import keyword
 import logging
 import os
 import pkgutil
-import re
 import signal
 import tempfile
 
 import cffi
 import numpy as np
-from numpy.lib.arraysetops import isin
+from numpy.lib.arraysetops import _setxor1d_dispatcher, isin
 
-from metview.dataset import FieldsetDb, Dataset
+from metview.metviewpy.indexdb import FieldsetDb
+from metview.dataset import Dataset
 from metview.style import (
     GeoView,
     Style,
@@ -33,8 +32,10 @@ from metview.style import (
     make_geoview,
 )
 from metview import plotting
+from metview.metviewpy.ipython import is_ipython_active, import_widgets
+from metview.metviewpy import utils
 
-__version__ = "1.8.1"
+__version__ = "1.10.2"
 
 
 # logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(message)s")
@@ -102,11 +103,14 @@ class MetviewInvoker:
         try:
             subprocess.Popen(metview_flags)
         except Exception as exp:  # pragma: no cover
-            print(
-                "Could not run the Metview executable ('" + metview_startup_cmd + "'); "
-                "check that the binaries for Metview (version 5 at least) are installed "
-                "and are in the PATH."
-            )
+            if "METVIEW_PYTHON_ONLY" not in os.environ:
+                print(
+                    "Could not run the Metview executable ('"
+                    + metview_startup_cmd
+                    + "'); "
+                    "check that the binaries for Metview (version 5 at least) are installed "
+                    "and are in the PATH."
+                )
             raise exp
 
         # wait for Metview to respond...
@@ -276,48 +280,56 @@ class Request(dict, Value):
     verb = "UNKNOWN"
 
     def __init__(self, req, myverb=None):
-        self.val_pointer = None
 
-        # initialise from Python object (dict/Request)
+        if isinstance(req, Request):  # copy an existing Request
+            self.xx = _request(req.get_verb(), req)  # to avoid deletion of Macro object
+            self.val_pointer = self.xx.val_pointer
+            self.verb = req.get_verb()
+            self.update(req)  # copy into dict
+            return
+
+        if myverb:
+            self.verb = myverb
         if isinstance(req, dict):
-            self.update(req)
-            self.to_metview_style()
-            if isinstance(req, Request):
-                self.verb = req.verb
-                self.val_pointer = req.val_pointer
-            else:
-                if myverb is not None:
-                    self.set_verb(myverb)
+            self.create_new(self.verb, req)
+            self.update(req)  # copy into dict
+            return
 
         # initialise from a Macro pointer
         else:
             Value.__init__(self, req)
             self.verb = string_from_ffi(lib.p_get_req_verb(req))
-            n = lib.p_get_req_num_params(req)
-            for i in range(0, n):
-                param = string_from_ffi(lib.p_get_req_param(req, i))
-                dict.__setitem__(self, param, self[param])
-            # self['_MACRO'] = 'BLANK'
-            # self['_PATH']  = 'BLANK'
+            super().update(self.to_dict())  # update dict
 
     def __str__(self):
         return "VERB: " + self.verb + super().__str__()
 
-    # translate Python classes into Metview ones where needed
-    def to_metview_style(self):
-        for k in list(self):
+    def to_dict(self):
+        keys = _keywords(self)
+        d = {}
+        for k in keys:
+            d[k] = self[k]
+        return d
 
-            # bool -> on/off
-            v = self.get(k)  # self[k] returns 1 for True
-            if isinstance(v, bool):
-                conversion_dict = {True: "on", False: "off"}
-                self[k] = conversion_dict[v]
+    # translate Python classes into Metview ones where needed - single parameter
+    def item_to_metview_style(self, key, value):
+        modified = False
+        delete_original_key = False
 
-            # class_ -> class (because 'class' is a Python keyword and cannot be
-            # used as a named parameter)
-            elif k == "class_":
-                self["class"] = v
-                del self["class_"]
+        # bool -> on/off
+        if isinstance(value, bool):
+            conversion_dict = {True: "on", False: "off"}
+            value = conversion_dict[value]
+            modified = True
+
+        # class_ -> class (because 'class' is a Python keyword and cannot be
+        # used as a named parameter)
+        elif key == "class_":
+            key = "class"
+            delete_original_key = True
+            modified = True
+
+        return (key, value, modified, delete_original_key)
 
     def set_verb(self, v):
         self.verb = v
@@ -325,23 +337,20 @@ class Request(dict, Value):
     def get_verb(self):
         return self.verb
 
+    def create_new(self, rverb, rdict):
+        r = definition()  # new, empty definition
+        self.val_pointer = r.val_pointer
+        self.xx = r  # to avoid deletion of Macro object
+        self.update(rdict)  # will set all parameters via __setitem__
+        return
+
     def push(self):
         # if we have a pointer to a Metview Value, then use that because it's more
         # complete than the dict
         if self.val_pointer:
             Value.push(self)
         else:
-            r = lib.p_new_request(self.verb.encode("utf-8"))
-
-            # to populate a request on the Macro side, we push each
-            # value onto its stack, and then tell it to create a new
-            # parameter with that name for the request. This allows us to
-            # use Macro to handle the addition of complex data types to
-            # a request
-            for k, v in self.items():
-                push_arg(v)
-                lib.p_set_request_value_from_pop(r, k.encode("utf-8"))
-
+            self.create_new(self.verb, self)
             lib.p_push_request(r)
 
     def update(self, items, sub=""):
@@ -366,9 +375,10 @@ class Request(dict, Value):
         return item
 
     def __setitem__(self, index, value):
-        if self.val_pointer and value is not None:
-            push_arg(index)
-            push_arg(value)
+        if (self.val_pointer) and (value is not None):
+            new_key, new_val, _, _ = self.item_to_metview_style(index, value)
+            push_arg(new_key)
+            push_arg(new_val)
             lib.p_set_subvalue_from_arg_stack(self.val_pointer)
         dict.__setitem__(self, index, value)
 
@@ -455,25 +465,6 @@ def valid_date(*args, base=None, step=None, step_units=None):
         return [base + step_units * int(x) for x in step]
 
 
-def get_file_list(path, file_name_pattern=None):
-    m = None
-    # if isinstance(file_name_pattern, re.Pattern):
-    #    m = file_name_pattern.match
-    # elif isinstance(file_name_pattern, str):
-    if isinstance(file_name_pattern, str):
-        if file_name_pattern.startswith('re"'):
-            m = re.compile(file_name_pattern[3:-1]).match
-
-    if m is not None:
-        return [
-            os.path.join(path, f) for f in builtins.filter(m, os.listdir(path=path))
-        ]
-    else:
-        if isinstance(file_name_pattern, str) and file_name_pattern != "":
-            path = os.path.join(path, file_name_pattern)
-        return sorted(glob.glob(path))
-
-
 class File(Value):
     def __init__(self, val_pointer):
         Value.__init__(self, val_pointer)
@@ -667,10 +658,10 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
             if isinstance(path, list):
                 v = []
                 for p in path:
-                    v.extend(get_file_list(p))
+                    v.extend(utils.get_file_list(p))
                 path = v
             else:
-                path = get_file_list(path)
+                path = utils.get_file_list(path)
 
             # fill the 'fields' var - it will be used a few lines down
             fields = [read(p) for p in path]
@@ -684,6 +675,7 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
         if self.val_pointer is not None:  #  we will overwrite ourselves, so delete
             lib.p_destroy_value(self.val_pointer)
         self.steal_val_pointer(temp)
+        self._db = None
 
     def to_dataset(self, **kwarg):
         # soft dependency on cfgrib
@@ -716,10 +708,10 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
         else:
             return self._db.select(**kwargs)
 
-    def describe(self, *args):
+    def describe(self, *args, **kwargs):
         if self._db is None:
             self._db = FieldsetDb(fs=self)
-        return self._db.describe(*args)
+        return self._db.describe(*args, **kwargs)
 
     def ls(self, **kwargs):
         if self._db is None:
@@ -741,6 +733,19 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
         else:
             return str()
 
+    @label.setter
+    def label(self, value):
+        if self._db is not None:
+            if self._db.label:
+                print(
+                    f"Warning: cannot set label! It is already set on index database as {self._db.label}!"
+                )
+                return
+            else:
+                self._db.label = value
+
+        self._label = value
+
     def unique(self, key):
         if self._db:
             return self._db.unique(key)
@@ -907,7 +912,7 @@ def dataset_to_fieldset(val, **kwarg):
     # we try to import xarray as locally as possible to reduce startup time
     # try to write the xarray as a GRIB file, then read into a fieldset
     import xarray as xr
-    import cfgrib
+    from cfgrib.xarray_to_grib import to_grib
 
     if not isinstance(val, xr.core.dataset.Dataset):
         raise TypeError(
@@ -921,7 +926,7 @@ def dataset_to_fieldset(val, **kwarg):
 
     try:
         # could add keys, e.g. grib_keys={'centre': 'ecmf'})
-        cfgrib.to_grib(val, tmp, **kwarg)
+        to_grib(val, tmp, **kwarg)
     except:
         print(
             "Error trying to write xarray dataset to GRIB for conversion to Metview Fieldset"
@@ -1221,6 +1226,7 @@ def bind_functions(namespace, module_name=None):
     # HACK: some functions are missing from the 'dictionary' call.
     namespace["neg"] = make("neg")
     namespace["nil"] = make("nil")
+    namespace["dialog"] = make("dialog")
     namespace["div"] = div
     namespace["mod"] = mod
     # override some functions that need special treatment
@@ -1232,18 +1238,24 @@ def bind_functions(namespace, module_name=None):
     namespace["merge"] = merge
     namespace["dataset_to_fieldset"] = dataset_to_fieldset
     namespace["valid_date"] = valid_date
-    namespace["get_file_list"] = get_file_list
     namespace["load_dataset"] = Dataset.load_dataset
     namespace["plot_maps"] = plotting.plot_maps
     namespace["plot_diff_maps"] = plotting.plot_diff_maps
     namespace["plot_xs"] = plotting.plot_xs
     namespace["plot_stamp"] = plotting.plot_stamp
+    namespace["plot_rmse"] = plotting.plot_rmse
+    namespace["plot_cdf"] = plotting.plot_cdf
     namespace["map_style_gallery"] = map_style_gallery
     namespace["map_area_gallery"] = map_area_gallery
     namespace["make_geoview"] = make_geoview
     namespace["Fieldset"] = Fieldset
     namespace["Request"] = Request
 
+    # some ui specific functions are prefixed with _. They will be exposed via the ui module!
+    for name in ["dialog", "any", "colour", "icon", "option_menu", "slider", "toggle"]:
+        namespace["_" + name] = namespace[name]
+        namespace.pop(name)
+
     # add some object methods the to global namespace
     for name in ["to_dataset", "to_dataframe", "ls", "describe", "select"]:
         namespace[name] = _make_function_for_object(name)
@@ -1253,12 +1265,14 @@ def bind_functions(namespace, module_name=None):
 add = make("+")
 call = make("call")
 count = make("count")
+definition = make("definition")
 div = make("/")
 download = make("download")
 equal = make("=")
 filter = make("filter")
 greater_equal_than = make(">=")
 greater_than = make(">")
+_keywords = make("keywords")
 lower_equal_than = make("<=")
 lower_than = make("<")
 met_merge = make("&")
@@ -1279,6 +1293,7 @@ met_and = make("and")
 met_or = make("or")
 met_not = make("not")
 met_version_info = make("version_info")
+_request = make("request")
 write = make("write")
 
 
@@ -1304,12 +1319,24 @@ def merge(*args):
 
 
 class Plot:
+    has_pillow = None
+    padding = np.array([x * 40 for x in [-1, -1, 1, 1]])
+
     def __init__(self):
         self.plot_to_jupyter = False
         self.plot_widget = True
         self.jupyter_args = {}
+        self.called_once = False
+        self.setoutput_called_once = False
 
     def __call__(self, *args, **kwargs):
+        # first time called? If we are in Jupyter and user did not specify,
+        # then plot inline to Jupyter by default
+        if not (self.called_once or self.setoutput_called_once):
+            if is_ipython_active():
+                setoutput("jupyter")
+        self.called_once = True
+
         if self.plot_to_jupyter:  # pragma: no cover
             if self.plot_widget:
                 return plot_to_notebook(args, **kwargs)
@@ -1330,6 +1357,42 @@ class Plot:
             # None is better for Python
             return None
 
+    def crop_image(self, path):
+        if Plot.has_pillow is None:
+            try:
+                import PIL
+
+                Plot.has_pillow = True
+            except ImportError as e:
+                Plot.has_pillow = False
+        if Plot.has_pillow:
+            try:
+                from PIL import Image
+                from PIL import ImageOps
+
+                im = Image.open(path)
+                im.load()
+
+                # find inner part
+                im_invert = im.convert("RGB")
+                box = ImageOps.invert(im_invert).getbbox()
+
+                # crop to box
+                if box[2] - box[0] > 100 or box[3] - box[1] > 100:
+                    box = list(np.asarray(box) + Plot.padding)
+                    box = (
+                        max(0, box[0]),
+                        max(0, box[1]),
+                        min(im.size[0], box[2]),
+                        min(im.size[1], box[3]),
+                    )
+                    im_crop = im.crop(box)
+                    im.close()
+                    im_crop.save(path)
+            except Exception as e:
+                # print(f"ERROR={e}")
+                pass
+
 
 plot = Plot()
 
@@ -1367,6 +1430,9 @@ def plot_to_notebook(*args, **kwargs):  # pragma: no cover
 
     files = [os.path.join(tempdirpath, f) for f in sorted(filenames)]
 
+    for f in files:
+        plot.crop_image(f)
+
     if (animation_mode == True) or (animation_mode == "auto" and len(filenames) > 1):
         frame_widget = widgets.IntSlider(
             value=1,
@@ -1444,6 +1510,8 @@ def plot_to_notebook_return_image(*args, **kwargs):  # pragma: no cover
     plot.jupyter_args.update(output_name=base, output_name_first_page_number="off")
     met_setoutput(png_output(plot.jupyter_args))
     met_plot(*args)
+    plot.crop_image(tmp)
+
     image = Image(tmp)
     os.unlink(tmp)
     return image
@@ -1454,19 +1522,16 @@ def plot_to_notebook_return_image(*args, **kwargs):  # pragma: no cover
 # functionality. Since this occurs within a function, we need a little trickery to
 # get the IPython functions into the global namespace so that the plot object can use them
 def setoutput(*args, **kwargs):
+    plot.setoutput_called_once = True
     if "jupyter" in args:  # pragma: no cover
-        try:
-            import IPython
-
-            get_ipython = IPython.get_ipython
-        except ImportError as imperr:
-            print("Could not import IPython module - plotting to Jupyter will not work")
-            raise imperr
-
-        # test whether we're in the Jupyter environment
-        if get_ipython() is not None:
+        if is_ipython_active():
+            global widgets
             plot.plot_to_jupyter = True
             plot.plot_widget = kwargs.get("plot_widget", True)
+            if plot.plot_widget:
+                widgets = import_widgets()
+                if not widgets:
+                    plot.plot_widget = False
             if "plot_widget" in kwargs:
                 del kwargs["plot_widget"]
             plot.jupyter_args = kwargs
@@ -1476,15 +1541,6 @@ def setoutput(*args, **kwargs):
             )
             raise (Exception("Could not set output to jupyter"))
 
-        try:
-            global widgets
-            widgets = __import__("ipywidgets", globals(), locals())
-        except ImportError as imperr:
-            print(
-                "Could not import ipywidgets module - plotting to Jupyter will not work"
-            )
-            raise imperr
-
     else:
         plot.plot_to_jupyter = False
         met_setoutput(*args)
