@@ -17,7 +17,8 @@ from pathlib import Path
 
 import yaml
 import metview as mv
-from metview.param import ParamInfo
+from metview.metviewpy.param import ParamInfo
+from metview.metviewpy.ipython import is_ipython_active
 
 # logging.basicConfig(level=logging.DEBUG)
 LOG = logging.getLogger(__name__)
@@ -208,7 +209,7 @@ class Style:
     def to_request(self):
         return [vd.to_request() for vd in self.visdefs]
 
-    def update(self, *args, inplace=None, verb=None):
+    def update(self, *args, inplace=False, verb=None):
         s = self if inplace == True else self.clone()
         if isinstance(verb, str):
             verb = [verb]
@@ -535,6 +536,7 @@ class MapConf:
     items = []
     areas = []
     BUILTIN_AREAS = [
+        "AFRICA",
         "ANTARCTIC",
         "ARCTIC",
         "AUSTRALASIA",
@@ -542,7 +544,6 @@ class MapConf:
         "CENTRAL_EUROPE",
         "EAST_TROPIC",
         "EASTERN_ASIA",
-        "EQUATORIAL_PACIFIC",
         "EURASIA",
         "EUROPE",
         "GLOBAL",
@@ -618,7 +619,10 @@ class MapConf:
             a = self.areas.get(area_v, {})
             if len(a) == 0 and area_v.upper() in self.BUILTIN_AREAS:
                 a = {"area_mode": "name", "area_name": area}
-        s = self.style_db.get_style(style_v)
+        if isinstance(style_v, mv.Request):
+            s = style_v
+        else:
+            s = self.style_db.get_style(style_v)
         return a, s
 
     def make_geo_view(self, area=None, style=None, plot_type=None):
@@ -626,22 +630,22 @@ class MapConf:
             style = "base_diff"
 
         a, s = self.find(area=area, style=style)
-        if s is not None:
-            s = s.clone()
-
-        if plot_type == "stamp":
-            s = s.update({"map_grid": "off", "map_label": "off"})
-
         if a is not None and a:
             a = copy.deepcopy(a)
         else:
             a = {}
 
-        if s is not None and s:
-            a["coastlines"] = s.to_request()
+        if s is not None:
+            if isinstance(s, mv.Request):
+                s = Visdef.from_request(s)
+            else:
+                s = s.clone()
+            if plot_type == "stamp":
+                s = s.update({"map_grid": "off", "map_label": "off"})
+            if s:
+                a["coastlines"] = s.to_request()
 
         return mv.geoview(**a)
-        # return GeoView(a, s)
 
 
 class StyleGallery:
@@ -654,11 +658,10 @@ class StyleGallery:
         with open(image_path, "rb") as img_file:
             return base64.b64encode(img_file.read()).decode("utf-8")
 
-    def build_gallery(self, names, images, row_height=None):
+    def build_gallery(self, names, images, row_height="150px"):
         figures = []
         # print(len(names))
         # print(len(images))
-        row_height = "150px" if row_height is None else row_height
         for name, image in zip(names, images):
             src = f"data:image/png;base64,{image}"
             caption = f'<figcaption style="text-align: center; font-size: 0.8em">{name}</figcaption>'
@@ -677,13 +680,7 @@ class StyleGallery:
         """
 
     def build(self):
-        try:
-            import IPython
-
-            # test whether we're in the Jupyter environment
-            if IPython.get_ipython() is None:
-                return
-        except:
+        if not is_ipython_active():
             return
 
         img_size = 120
@@ -785,9 +782,8 @@ def make_eccharts_mcont():
     return mv.mcont(contour_automatic_settings="ecmwf", legend="on")
 
 
-def get_db(name=None):
+def get_db(name="param"):
     global _DB
-    name = "param" if name is None else name
     assert name in _DB
     if _DB[name][0] is None:
         _DB[name][0] = StyleDb(_DB[name][1], _DB[name][2])
@@ -803,10 +799,9 @@ def find(name):
         return None
 
 
-def load_custom_config(conf_dir, force=None):
+def load_custom_config(conf_dir, force=False):
     global CUSTOM_CONF_PATH
     global _MAP_CONF
-    force = False if force is None else force
     if CUSTOM_CONF_PATH:
         if CUSTOM_CONF_PATH[-1] == conf_dir:
             if force:
